"""The module describes the ``env`` job metrics plugin."""
import logging
import re

from . import InstrumentPlugin
from .. import formatting

log = logging.getLogger(__name__)


class EnvFormatter(formatting.JobMetricFormatter):
    pass


class EnvPlugin(InstrumentPlugin):
    """ Instrumentation plugin capable of recording all or specific environment
    variables for a job at runtime.
    """
    plugin_type = "env"
    formatter = EnvFormatter()

    def __init__(self, **kwargs):
        variables_str = kwargs.get("variables", None)
        if variables_str:
            variables = [v.strip() for v in variables_str.split(",")]
        else:
            variables = None
        self.variables = variables

    def pre_execute_instrument(self, job_directory):
        """ Use env to dump all environment variables to a file.
        """
        return f"env > '{self.__env_file(job_directory)}'"

    def post_execute_instrument(self, job_directory):
        return None

    def job_properties(self, job_id, job_directory):
        """ Recover environment variables dumped out on compute server and filter
        out specific variables if needed.
        """
        variables = self.variables

        properties = {}
        env_string = ''.join(open(self.__env_file(job_directory)).readlines())
        while env_string:
            # Check if the next lines contain a shell function.
            # We use '\n\}\n' as regex termination because shell
            # functions can be nested.
            # We use the non-greedy '.+?' because of re.DOTALL .
            m = re.match(r'([^=]+)=(\(\) \{.+?\n\})\n', env_string, re.DOTALL)
            if m is None:
                m = re.match('([^=]+)=(.*)\n', env_string)
            if m is None:
                # Some problem recording or reading back env output.
                message_template = "Problem parsing env metric output for job %s - properties will be incomplete"
                message = message_template % job_id
                log.debug(message)
                break
            (var, value) = m.groups()
            if not variables or var in variables:
                properties[var] = value
            env_string = env_string[m.end():]

        return properties

    def __env_file(self, job_directory):
        return self._instrument_file_path(job_directory, "vars")


__all__ = ('EnvPlugin', )



"""The module describes the ``hostname`` job metrics plugin."""
import logging

from . import InstrumentPlugin
from .. import formatting

log = logging.getLogger(__name__)


class HostnameFormatter(formatting.JobMetricFormatter):

    def format(self, key, value):
        return key, value


class HostnamePlugin(InstrumentPlugin):
    """ Gather hostname
    """
    plugin_type = "hostname"
    formatter = HostnameFormatter()

    def __init__(self, **kwargs):
        pass

    def pre_execute_instrument(self, job_directory):
        return f"hostname -f > '{self.__instrument_hostname_path(job_directory)}'"

    def job_properties(self, job_id, job_directory):
        with open(self.__instrument_hostname_path(job_directory)) as f:
            return {'hostname': f.read().strip()}

    def __instrument_hostname_path(self, job_directory):
        return self._instrument_file_path(job_directory, "hostname")


__all__ = ('HostnamePlugin', )



"""The module describes the ``uname`` job metrics plugin."""
from . import InstrumentPlugin
from .. import formatting


class UnameFormatter(formatting.JobMetricFormatter):

    def format(self, key, value):
        return "Operating System", value


class UnamePlugin(InstrumentPlugin):
    """ Use uname to gather operating system information about remote system
    job is running on. Linux only.
    """
    plugin_type = "uname"
    formatter = UnameFormatter()

    def __init__(self, **kwargs):
        self.uname_args = kwargs.get("args", "-a")

    def pre_execute_instrument(self, job_directory):
        return f"uname {self.uname_args} > '{self.__instrument_uname_path(job_directory)}'"

    def job_properties(self, job_id, job_directory):
        properties = {}
        with open(self.__instrument_uname_path(job_directory)) as f:
            properties["uname"] = f.read()
        return properties

    def __instrument_uname_path(self, job_directory):
        return self._instrument_file_path(job_directory, "uname")


__all__ = ('UnamePlugin', )



"""This module describes :class:`CollectlCli` - an abstraction for building collectl command lines."""
import logging
import subprocess
from string import Template

log = logging.getLogger(__name__)

COMMAND_LINE_TEMPLATE = Template(
    "$collectl_path $destination_arg $mode_arg $subsystems_arg $interval_arg $procfilt_arg $flush_arg $sep_arg"
)
MODE_RECORD = "record"
MODE_PLAYBACK = "playback"


class CollectlCli:
    """
    Abstraction over (some of) the command-line arguments of collectl.
    Ideally this will be useful for building up command line arguments for
    remote execution as well as runnning directly on local host.

    This is meant to be a fairly generic utility - for interfacing with
    collectl CLI - logic more directly related to the Galaxy job metric plugin
    plugin should be placed in other modules.

    **Keyword Arguments:**

    ``collectl_path``
        Path to collectl executable (defaults to collectl - i.e.
        search the PATH).

    ``playback_path`` (defaults to ``None``)
        If this is ``None``, collectl will run in
        record mode, else it will playback specified file.

    **Playback Mode Options:**

    ``sep``
        Separator used in playback mode (set to 9 to produce tsv)
        (defaults to None).

    **Record Mode Options** (some of these may work in playback mode also)

    ``destination_path``
        Location of path files to write to (defaults to None
        and collectl will just use cwd). Really this is just to prefix -
        collectl will append hostname and datetime to file.
    ``interval``
        Setup polling interval (secs) for most subsystems (defaults
        to None and when unspecified collectl will use default of 1 second).
    ``interval2``
        Setup polling interval (secs) for process information
        (defaults to None and when unspecified collectl will use default to
        60 seconds).
    ``interval3``
        Setup polling interval (secs) for environment information
        (defaults to None and when unspecified collectl will use default to
        300 seconds).
    ``procfilt``
        Optional argument to procfilt. (defaults to None).
    ``flush``
        Optional flush interval (defaults to None).
    """

    def __init__(self, **kwargs):
        command_args = {}
        command_args["collectl_path"] = kwargs.get("collectl_path", "collectl")
        playback_path = kwargs.get("playback_path", None)
        self.mode = MODE_RECORD if not playback_path else MODE_PLAYBACK
        if self.mode == MODE_RECORD:
            mode_arg = ""
        elif self.mode == MODE_PLAYBACK:
            mode_arg = f"-P -p '{playback_path}'"
        else:
            raise Exception(f"Invalid mode supplied to CollectlCli - {self.mode}")
        command_args["mode_arg"] = mode_arg
        command_args["interval_arg"] = self.__interval_arg(kwargs)
        destination = kwargs.get("destination_path", None)
        if destination:
            destination_arg = f"-f '{destination}'"
        else:
            destination_arg = ""
        command_args["destination_arg"] = destination_arg
        procfilt = kwargs.get("procfilt", None)
        command_args["procfilt_arg"] = "" if not procfilt else f"--procfilt {procfilt}"
        command_args["subsystems_arg"] = self.__subsystems_arg(kwargs.get("subsystems", []))
        flush = kwargs.get("flush", None)
        command_args["flush_arg"] = f"--flush {flush}" if flush else ""
        sep = kwargs.get("sep", None)
        command_args["sep_arg"] = f"--sep={sep}" if sep else ""

        self.command_args = command_args

    def __subsystems_arg(self, subsystems):
        if subsystems:
            return f"-s{''.join(s.command_line_arg for s in subsystems)}"
        else:
            return ""

    def __interval_arg(self, kwargs):
        if self.mode != MODE_RECORD:
            return ""

        interval = kwargs.get("interval", None)
        if not interval:
            return ""

        self.__validate_interval_arg(interval)
        interval_arg = f"-i {interval}"
        interval2 = kwargs.get("interval2", None)
        if not interval2:
            return interval_arg
        self.__validate_interval_arg(interval2, multiple_of=int(interval))
        interval_arg = f"{interval_arg}:{interval2}"

        interval3 = kwargs.get("interval3", None)
        if not interval3:
            return interval_arg
        self.__validate_interval_arg(interval3, multiple_of=int(interval))
        interval_arg = f"{interval_arg}:{interval3}"
        return interval_arg

    def __validate_interval_arg(self, value, multiple_of=None):
        if value and not str(value).isdigit():
            raise Exception(f"Invalid interval argument supplied, must be integer {value}")
        if multiple_of:
            if int(value) % multiple_of != 0:
                raise Exception(f"Invalid interval argument supplied, must multiple of {multiple_of}")

    def build_command_line(self):
        return COMMAND_LINE_TEMPLATE.substitute(**self.command_args)

    def run(self, stdout=subprocess.PIPE, stderr=subprocess.PIPE):
        command_line = self.build_command_line()
        log.info(f"Executing {command_line}")
        proc = subprocess.Popen(command_line, shell=True, stdout=stdout, stderr=stderr)
        return_code = proc.wait()
        if return_code:
            raise Exception("Problem running collectl command.")


__all__ = ('CollectlCli', )



"""The module describes the ``cpuinfo`` job metrics plugin."""
import logging
import re

from galaxy import util
from . import InstrumentPlugin
from .. import formatting

log = logging.getLogger(__name__)

PROCESSOR_LINE = re.compile(r"processor\s*\:\s*(\d+)")


class CpuInfoFormatter(formatting.JobMetricFormatter):

    def format(self, key, value):
        if key == "processor_count":
            return "Processor Count", f"{int(value)}"
        else:
            return key, value


class CpuInfoPlugin(InstrumentPlugin):
    """ Gather information about processor configuration from /proc/cpuinfo.
    Linux only.
    """
    plugin_type = "cpuinfo"
    formatter = CpuInfoFormatter()

    def __init__(self, **kwargs):
        self.verbose = util.asbool(kwargs.get("verbose", False))

    def pre_execute_instrument(self, job_directory):
        return f"cat /proc/cpuinfo > '{self.__instrument_cpuinfo_path(job_directory)}'"

    def job_properties(self, job_id, job_directory):
        properties = {}
        processor_count = 0
        with open(self.__instrument_cpuinfo_path(job_directory)) as f:
            current_processor = None
            for line in f:
                line = line.strip().lower()
                if not line:  # Skip empty lines
                    continue

                processor_line_match = PROCESSOR_LINE.match(line)
                if processor_line_match:
                    processor_count += 1
                    current_processor = processor_line_match.group(1)
                elif current_processor and self.verbose:
                    # If verbose, dump information about each processor
                    # into database...
                    key, value = line.split(":", 1)
                    key = f"processor_{current_processor}_{key.strip()}"
                    value = value
        properties["processor_count"] = processor_count
        return properties

    def __instrument_cpuinfo_path(self, job_directory):
        return self._instrument_file_path(job_directory, "cpuinfo")


__all__ = ('CpuInfoPlugin', )



"""The module describes the ``cgroup`` job metrics plugin."""
import logging
import numbers
from collections import namedtuple

from galaxy.util import asbool, nice_size
from . import InstrumentPlugin
from .. import formatting

log = logging.getLogger(__name__)

TITLES = {
    "memory.memsw.max_usage_in_bytes": "Max memory usage (MEM+SWP)",
    "memory.max_usage_in_bytes": "Max memory usage (MEM)",
    "memory.limit_in_bytes": "Memory limit on cgroup (MEM)",
    "memory.memsw.limit_in_bytes": "Memory limit on cgroup (MEM+SWP)",
    "memory.soft_limit_in_bytes": "Memory softlimit on cgroup",
    "memory.failcnt": "Failed to allocate memory count",
    "memory.oom_control.oom_kill_disable": "OOM Control enabled",
    "memory.oom_control.under_oom": "Was OOM Killer active?",
    "cpuacct.usage": "CPU Time"
}
CONVERSION = {
    "memory.oom_control.oom_kill_disable": lambda x: "No" if x == 1 else "Yes",
    "memory.oom_control.under_oom": lambda x: "Yes" if x == 1 else "No",
    "cpuacct.usage": lambda x: formatting.seconds_to_str(x / 10**9)  # convert nanoseconds
}
CPU_USAGE_TEMPLATE = r"""
if [ -e "/proc/$$/cgroup" -a -d "{cgroup_mount}" ]; then
    cgroup_path=$(cat "/proc/$$/cgroup" | awk -F':' '($2=="cpuacct,cpu") || ($2=="cpu,cpuacct") {{print $3}}');
    if [ ! -e "{cgroup_mount}/cpu$cgroup_path/cpuacct.usage" ]; then
        cgroup_path="";
    fi;
    for f in {cgroup_mount}/{{cpu\,cpuacct,cpuacct\,cpu}}$cgroup_path/{{cpu,cpuacct}}.*; do
        if [ -f "$f" ]; then
            echo "__$(basename $f)__" >> {metrics}; cat "$f" >> {metrics} 2>/dev/null;
        fi;
    done;
fi
""".replace("\n", " ").strip()
MEMORY_USAGE_TEMPLATE = """
if [ -e "/proc/$$/cgroup" -a -d "{cgroup_mount}" ]; then
    cgroup_path=$(cat "/proc/$$/cgroup" | awk -F':' '$2=="memory"{{print $3}}');
    if [ ! -e "{cgroup_mount}/memory$cgroup_path/memory.max_usage_in_bytes" ]; then
        cgroup_path="";
    fi;
    for f in {cgroup_mount}/memory$cgroup_path/memory.*; do
        echo "__$(basename $f)__" >> {metrics}; cat "$f" >> {metrics} 2>/dev/null;
    done;
fi
""".replace("\n", " ").strip()


Metric = namedtuple("Metric", ("key", "subkey", "value"))


class CgroupPluginFormatter(formatting.JobMetricFormatter):

    def format(self, key, value):
        title = TITLES.get(key, key)
        if key in CONVERSION:
            return title, CONVERSION[key](value)
        elif key.endswith("_bytes"):
            try:
                return title, nice_size(value)
            except ValueError:
                pass
        elif isinstance(value, numbers.Number) and value == int(value):
            value = int(value)
        return title, value


class CgroupPlugin(InstrumentPlugin):
    """ Plugin that collects memory and cpu utilization from within a cgroup.
    """
    plugin_type = "cgroup"
    formatter = CgroupPluginFormatter()

    def __init__(self, **kwargs):
        self.verbose = asbool(kwargs.get("verbose", False))
        self.cgroup_mount = kwargs.get("cgroup_mount", "/sys/fs/cgroup")
        params_str = kwargs.get("params", None)
        if params_str:
            params = [v.strip() for v in params_str.split(",")]
        else:
            params = TITLES.keys()
        self.params = params

    def post_execute_instrument(self, job_directory):
        commands = []
        commands.append(self.__record_cgroup_cpu_usage(job_directory))
        commands.append(self.__record_cgroup_memory_usage(job_directory))
        return commands

    def job_properties(self, job_id, job_directory):
        metrics = self.__read_metrics(self.__cgroup_metrics_file(job_directory))
        return metrics

    def __record_cgroup_cpu_usage(self, job_directory):
        # comounted cgroups (which cpu and cpuacct are on the supported Linux distros) can appear in any order (cpu,cpuacct or cpuacct,cpu)
        return CPU_USAGE_TEMPLATE.format(metrics=self.__cgroup_metrics_file(job_directory), cgroup_mount=self.cgroup_mount)

    def __record_cgroup_memory_usage(self, job_directory):
        return MEMORY_USAGE_TEMPLATE.format(metrics=self.__cgroup_metrics_file(job_directory), cgroup_mount=self.cgroup_mount)

    def __cgroup_metrics_file(self, job_directory):
        return self._instrument_file_path(job_directory, "_metrics")

    def __read_metrics(self, path):
        metrics = {}
        key = None
        with open(path) as infile:
            for line in infile:
                try:
                    metric, key = self.__read_key_value(line.strip(), key)
                except Exception:
                    log.exception("Caught exception attempting to read metric from cgroup line: %s", line)
                    metric = None
                if not metric:
                    continue
                self.__add_metric(metrics, metric)
        return metrics

    def __add_metric(self, metrics, metric):
        if metric and (metric.subkey in self.params or self.verbose):
            metrics[metric.subkey] = metric.value

    def __read_key_value(self, line, key):
        if line.startswith('__') and line.endswith('__'):
            # line is the beginning of a new param
            key = line[2:][:-2]
            return (None, key)
        elif line.count(" ") == 1:
            # line has a subkey
            subkey, value = line.split(" ", 1)
            subkey = ".".join((key, subkey))
        else:
            # line does not have a subkey
            subkey = key
            value = line
        value = self.__type_value(value)
        return (Metric(key, subkey, value), key)

    def __type_value(self, value):
        try:
            try:
                return int(value)
            except ValueError:
                return float(value)
        except ValueError:
            return value


__all__ = ('CgroupPlugin', )



"""The module describes the ``collectl`` job metrics plugin."""
import logging
import os
import shutil

from galaxy import util
from . import InstrumentPlugin
from .. import formatting
from ..collectl import (
    cli,
    processes,
    subsystems
)

log = logging.getLogger(__name__)

# By default, only grab statistics for user processes (as identified by
# username).
DEFAULT_PROCFILT_ON = "username"
DEFAULT_SUBSYSTEMS = "process"
# Set to zero to flush every collection.
DEFAULT_FLUSH_INTERVAL = "0"

FORMATTED_RESOURCE_TITLES = {
    "PCT": "Percent CPU Usage",
    "RSYS": "Disk Reads",
    "WSYS": "Disk Writes",
}

EMPTY_COLLECTL_FILE_MESSAGE = "Skipping process summary due to empty file... job probably did not run long enough for collectl to gather data."


class CollectlFormatter(formatting.JobMetricFormatter):

    def format(self, key, value):
        if key == "pid":
            return ("Process ID", int(value))
        elif key == "raw_log_path":
            return ("Relative Path of Full Collectl Log", value)
        elif key == "process_max_AccumT":
            return ("Job Runtime (System+User)", formatting.seconds_to_str(float(value)))
        else:
            _, stat_type, resource_type = key.split("_", 2)
            if resource_type.startswith("Vm"):
                value_str = f"{int(value)} KB"
            elif resource_type in ["RSYS", "WSYS"] and stat_type in ["count", "max", "sum"]:
                value_str = "%d (# system calls)" % int(value)
            else:
                value_str = str(value)
            resource_title = FORMATTED_RESOURCE_TITLES.get(resource_type, resource_type)
            return (f"{resource_title} ({stat_type})", value_str)


class CollectlPlugin(InstrumentPlugin):
    """ Run collectl along with job to capture system and/or process data
    according to specified collectl subsystems.
    """
    plugin_type = "collectl"
    formatter = CollectlFormatter()

    def __init__(self, **kwargs):
        self.__configure_paths(kwargs)
        self.__configure_subsystems(kwargs)
        saved_logs_path = kwargs.get("saved_logs_path", "")
        if "app" in kwargs:
            log.debug(f"Found path for saved logs: {saved_logs_path}")
            saved_logs_path = kwargs["app"].config.resolve_path(saved_logs_path)
        self.saved_logs_path = saved_logs_path
        self.__configure_collectl_recorder_args(kwargs)
        self.summarize_process_data = util.asbool(kwargs.get("summarize_process_data", True))
        self.log_collectl_program_output = util.asbool(kwargs.get("log_collectl_program_output", False))
        if self.summarize_process_data:
            if subsystems.get_subsystem("process") not in self.subsystems:
                raise Exception("Collectl plugin misconfigured - cannot summarize_process_data without process subsystem being enabled.")

            process_statistics = kwargs.get("process_statistics", None)
            # None will let processes module use default set of statistics
            # defined there.
            self.process_statistics = processes.parse_process_statistics(process_statistics)

    def pre_execute_instrument(self, job_directory):
        commands = []
        # Capture PID of process so we can walk its ancestors when building
        # statistics for the whole job.
        commands.append(f'''echo "$$" > '{self.__pid_file(job_directory)}' ''')
        # Run collectl in record mode to capture process and system level
        # statistics according to supplied subsystems.
        commands.append(self.__collectl_record_command(job_directory))
        return commands

    def post_execute_instrument(self, job_directory):
        commands = []
        # collectl dies when job script completes, perhaps capture pid of
        # collectl above and check if it is still alive to allow tracking if
        # collectl ran successfully through the whole job.
        return commands

    def job_properties(self, job_id, job_directory):
        pid = open(self.__pid_file(job_directory)).read().strip()
        contents = os.listdir(job_directory)
        try:
            rel_path = next(iter(filter(self._is_instrumented_collectl_log, contents)))
            path = os.path.join(job_directory, rel_path)
        except IndexError:
            message = f"Failed to find collectl log in directory {job_directory}, files were {contents}"
            raise Exception(message)

        properties = dict(
            pid=int(pid),
        )

        if self.saved_logs_path:
            destination_rel_dir = os.path.join(*util.directory_hash_id(job_id))
            destination_rel_path = os.path.join(destination_rel_dir, rel_path)
            destination_path = os.path.join(self.saved_logs_path, destination_rel_path)
            destination_dir = os.path.dirname(destination_path)
            if not os.path.isdir(destination_dir):
                os.makedirs(destination_dir)
            shutil.copyfile(path, destination_path)
            properties["raw_log_path"] = destination_rel_path

        if self.summarize_process_data:
            # Run collectl in playback and generate statistics of interest
            summary_statistics = self.__summarize_process_data(pid, path)
            for statistic, value in summary_statistics:
                properties[f"process_{'_'.join(statistic)}"] = value

        return properties

    def __configure_paths(self, kwargs):
        # 95% of time I would expect collectl to just be installed with apt or
        # yum, but if it is manually installed on not on path, allow
        # configuration of explicit path - and allow path to be different
        # between galaxy job handler (local_collectl_path) and compute node
        # (remote_collectl_path).
        collectl_path = kwargs.get("collectl_path", "collectl")
        self.remote_collectl_path = kwargs.get("remote_collectl_path", collectl_path)
        self.local_collectl_path = kwargs.get("local_collectl_path", collectl_path)

    def __configure_subsystems(self, kwargs):
        raw_subsystems_str = kwargs.get("subsystems", DEFAULT_SUBSYSTEMS)
        raw_subsystems = util.listify(raw_subsystems_str, do_strip=True)
        self.subsystems = [subsystems.get_subsystem(_) for _ in raw_subsystems]

    def __configure_collectl_recorder_args(self, kwargs):
        collectl_recorder_args = kwargs.copy()

        # Allow deployer to configure separate system and process intervals,
        # but if they specify just one - use it for both. Thinking here is this
        # plugin's most useful feature is the process level information so
        # this is likely what the deployer is attempting to configure.
        if "interval" in kwargs and "interval2" not in kwargs:
            collectl_recorder_args["interval2"] = kwargs["interval"]

        if "flush" not in kwargs:
            collectl_recorder_args["flush"] = DEFAULT_FLUSH_INTERVAL

        procfilt_on = kwargs.get("procfilt_on", DEFAULT_PROCFILT_ON).lower()
        # Calculate explicit arguments, rest can just be passed through from
        # constructor arguments.
        explicit_args = dict(
            collectl_path=self.remote_collectl_path,
            procfilt=procfilt_argument(procfilt_on),
            subsystems=self.subsystems,
        )
        collectl_recorder_args.update(explicit_args)
        self.collectl_recorder_args = collectl_recorder_args

    def __summarize_process_data(self, pid, collectl_log_path):
        playback_cli_args = dict(
            collectl_path=self.local_collectl_path,
            playback_path=collectl_log_path,
            sep="9"
        )
        if not os.stat(collectl_log_path).st_size:
            log.debug(EMPTY_COLLECTL_FILE_MESSAGE)
            return []

        playback_cli = cli.CollectlCli(**playback_cli_args)
        return processes.generate_process_statistics(playback_cli, pid, self.process_statistics)

    def __collectl_recorder_cli(self, job_directory):
        cli_args = self.collectl_recorder_args.copy()
        cli_args["destination_path"] = self._instrument_file_path(job_directory, "log")
        return cli.CollectlCli(**cli_args)

    def __collectl_record_command(self, job_directory):
        collectl_cli = self.__collectl_recorder_cli(job_directory)
        if self.log_collectl_program_output:
            redirect_to = self._instrument_file_path(job_directory, "program_output")
        else:
            redirect_to = "/dev/null"
        return f"{collectl_cli.build_command_line()} > {redirect_to} 2>&1 &"

    def __pid_file(self, job_directory):
        return self._instrument_file_path(job_directory, "pid")

    def _is_instrumented_collectl_log(self, filename):
        prefix = self._instrument_file_name("log")
        return filename.startswith(prefix) and filename.endswith(".raw.gz")


def procfilt_argument(procfilt_on):
    if procfilt_on == "username":
        return "U$USER"
    elif procfilt_on == "uid":
        return "u$UID"
    else:
        # Ensure it is empty of None
        if procfilt_on or procfilt_on.lower() != "none":
            raise Exception("Invalid procfilt_on argument encountered")
        return ""


__all__ = ('CollectlPlugin', )



__version__ = "22.1.1"

PROJECT_NAME = "galaxy-job-metrics"
PROJECT_OWNER = PROJECT_USERAME = "galaxyproject"
PROJECT_URL = "https://github.com/galaxyproject/galaxy"
PROJECT_AUTHOR = 'Galaxy Project and Community'
PROJECT_DESCRIPTION = 'Galaxy Job Metrics'
PROJECT_EMAIL = 'galaxy-committers@lists.galaxyproject.org'
RAW_CONTENT_URL = "https://raw.github.com/{}/{}/master/".format(
    PROJECT_USERAME, PROJECT_NAME
)



"""Utilities related to formatting job metrics for human consumption."""


class JobMetricFormatter:
    """Format job metric key-value pairs for human consumption in Web UI."""

    def format(self, key, value):
        return (str(key), str(value))


def seconds_to_str(value):
    """Convert seconds to a simple simple string describing the amount of time."""
    mins, secs = divmod(value, 60)
    hours, mins = divmod(mins, 60)

    if value < 60:
        return f"{secs} second{'s' if secs != 1 else ''}"
    elif value < 3600:
        return f"{mins} minute{'s' if mins != 1 else ''}"
    else:
        return f"{hours} hour{'s' if hours != 1 else ''} and {mins} minute{'s' if mins != 1 else ''}"



from galaxy.job_metrics import (
    formatting,
    JobMetrics,
)


def test_job_metrics_load():
    # Just construct the manager to make sure all the plugin classes load fine
    # and package is configured properly.
    JobMetrics()


def test_job_metric_formatting():
    assert formatting.seconds_to_str(0) == "0 seconds"
    assert formatting.seconds_to_str(1) == "1 second"
    assert formatting.seconds_to_str(59) == "59 seconds"
    assert formatting.seconds_to_str(60) == "1 minute"
    assert formatting.seconds_to_str(61) == "1 minute"
    assert formatting.seconds_to_str(119) == "1 minute"
    assert formatting.seconds_to_str(120) == "2 minutes"
    assert formatting.seconds_to_str(121) == "2 minutes"
    assert formatting.seconds_to_str(600) == "10 minutes"
    assert formatting.seconds_to_str(601) == "10 minutes"
    assert formatting.seconds_to_str(660) == "11 minutes"
    assert formatting.seconds_to_str(661) == "11 minutes"
    assert formatting.seconds_to_str(3600) == "1 hour and 0 minutes"
    assert formatting.seconds_to_str(3601) == "1 hour and 0 minutes"
    assert formatting.seconds_to_str(3660) == "1 hour and 1 minute"
    assert formatting.seconds_to_str(3661) == "1 hour and 1 minute"
    assert formatting.seconds_to_str(7260) == "2 hours and 1 minute"
    assert formatting.seconds_to_str(7320) == "2 hours and 2 minutes"
    assert formatting.seconds_to_str(36181) == "10 hours and 3 minutes"
